/*
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
 *
 * The contents of this file are subject to the terms of either the GNU
 * General Public License Version 2 only ("GPL") or the Common
 * Development and Distribution License("CDDL") (collectively, the
 * "License"). You may not use this file except in compliance with the
 * License. You can obtain a copy of the License at
 * http://www.netbeans.org/cddl-gplv2.html
 * or nbbuild/licenses/CDDL-GPL-2-CP. See the License for the
 * specific language governing permissions and limitations under the
 * License.  When distributing the software, include this License Header
 * Notice in each file and include the License file at
 * nbbuild/licenses/CDDL-GPL-2-CP.  Sun designates this
 * particular file as subject to the "Classpath" exception as provided
 * by Sun in the GPL Version 2 section of the License file that
 * accompanied this code. If applicable, add the following below the
 * License Header, with the fields enclosed by brackets [] replaced by
 * your own identifying information:
 * "Portions Copyrighted [year] [name of copyright owner]"
 *
 * Contributor(s):
 *
 * Portions Copyrighted 2009 Jaroslav Tulach
 */

package cz.xelfi.quoridor.webidor.resources;

import com.sun.jersey.api.container.httpserver.HttpServerFactory;
import com.sun.jersey.api.core.PackagesResourceConfig;
import com.sun.jersey.api.core.ResourceConfig;
import com.sun.jersey.api.json.JSONWithPadding;
import com.sun.jersey.spi.resource.Singleton;
import com.sun.net.httpserver.HttpServer;
import cz.xelfi.quoridor.webidor.User;
import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.net.ServerSocket;
import java.util.HashMap;
import java.util.Map;
import java.util.Properties;
import java.util.UUID;
import javax.ws.rs.GET;
import javax.ws.rs.PUT;
import javax.ws.rs.Path;
import javax.ws.rs.Produces;
import javax.ws.rs.QueryParam;
import javax.ws.rs.WebApplicationException;
import javax.ws.rs.core.MediaType;
import javax.ws.rs.core.Response.Status;

/**
 *
 * @author Jaroslav Tulach <jtulach@netbeans.org>
 */
@Path("/")
@Singleton
public final class Quoridor {
    private final File path;
    private Games games;
    private Users users;
    private final Map<UUID,String> loggedIn;

    public Quoridor() {
        final String prop = System.getProperty("quoridor.dir"); // NOI18N
        if (prop == null) {
            throw new IllegalStateException("quoridor.dir property must be specified"); // NOI18N
        }
        path = new File(prop);
        path.mkdirs();
        loggedIn = new HashMap<UUID, String>();
        games = new Games(new File(path, "games"), this); // NOI18N
        users = new Users(new File(path, "users"), this); // NOI18N
    }

    @Path("games")
    public Games getGames() {
        return games;
    }

    @Path("users")
    public Users getUsers() {
        return users;
    }

    @Path("login")
    @PUT
    @Produces({ MediaType.TEXT_PLAIN })
    public String login(
        @QueryParam("name") String name,
        @QueryParam("password") String password
    ) {
        File f = new File(path, "passwd"); // NOI18Nt
        Properties p = new Properties();
        try {
            p.load(new FileInputStream(f));
        } catch (IOException ex) {
            ex.printStackTrace();
        }
        if (name != null && password.equals(p.getProperty(name))) {
            UUID uuid = UUID.randomUUID();
            loggedIn.put(uuid, name);
            return uuid.toString();
        } else {
            return null;
        }
        
    }

    @Path("login")
    @GET
    @Produces({ MediaType.TEXT_PLAIN })
    public String isLoggedIn(
        @QueryParam("id") String id
    ) {
        String ret = null;
        try {
            if (id != null) {
                ret = loggedIn.get(UUID.fromString(id));
            }
        } catch (IllegalArgumentException ex) {
            // OK, happens for invalid ids
        }
        return ret == null ? "" : ret;
    }

    @Path("login")
    @GET
    @Produces({ "application/x-javascript", MediaType.APPLICATION_JSON, MediaType.TEXT_XML })
    public JSONWithPadding loggedInInfo(
        @QueryParam("callback") String callback,
        @QueryParam("id") String id
    ) throws IOException {
        String ret = null;
        try {
            if (id != null) {
                ret = loggedIn.get(UUID.fromString(id));
            }
        } catch (IllegalArgumentException ex) {
            // OK, happens for invalid ids
        }
        if (ret == null) {
            throw new WebApplicationException(Status.UNAUTHORIZED);
        } else {
            return getUsers().getUserInfo(callback, id, ret);
        }
    }

    //
    // start the server
    //

    public static void main(String[] args) throws IOException, InterruptedException {
        int port = 9222;
        try {
            port = Integer.parseInt(args[0]);
        } catch (Exception ex) {
            // OK
        }
        HttpServer s = start(port);
        System.out.println(
            "Quoridor started at port " + port + "\n" + "Hit enter to stop it..."
        );
        if (args.length < 2 || !args[args.length - 1].equals("--kill")) {
            System.out.println("Hit enter to stop it...");
            System.in.read();
        } else {
            synchronized (Quoridor.class) {
                Quoridor.class.wait();
            }
        }
        System.in.read();
        s.stop(0);
        System.exit(0);
    }

    public static HttpServer start(int port) throws IOException {
        if (port == -1) {
            ServerSocket ss = new ServerSocket(0);
            port =ss.getLocalPort();
            ss.close();
        }
        final String baseUri = "http://localhost:" + port + "/";

        if (System.getProperty("quoridor.dir") == null) {
            File home = new File(System.getProperty("user.home"));
            File quoridor = new File(home, ".quoridor");
            System.setProperty("quoridor.dir", quoridor.getPath());
        }

        ResourceConfig rc = new PackagesResourceConfig("cz.xelfi.quoridor.webidor");
        HttpServer server = HttpServerFactory.create(baseUri, rc);
        server.start();
        return server;
    }

}
